# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from senlin.common import utils as common_utils
from senlin.engine import senlin_lock as lockm
from senlin.objects import action as ao
from senlin.objects import cluster_lock as clo
from senlin.objects import node_lock as nlo
from senlin.objects import service as svco
from senlin.tests.unit.common import base
from senlin.tests.unit.common import utils


class SenlinLockTest(base.SenlinTestCase):

    def setUp(self):
        super(SenlinLockTest, self).setUp()

        self.ctx = utils.dummy_context()

        ret = mock.Mock(owner='ENGINE', id='ACTION_ABC')
        self.stub_get = self.patchobject(ao.Action, 'get', return_value=ret)

    @mock.patch.object(clo.ClusterLock, "acquire")
    def test_cluster_lock_acquire_already_owner(self, mock_acquire):
        mock_acquire.return_value = ['ACTION_XYZ']

        res = lockm.cluster_lock_acquire(self.ctx, 'CLUSTER_A', 'ACTION_XYZ')

        self.assertTrue(res)
        mock_acquire.assert_called_once_with('CLUSTER_A', 'ACTION_XYZ',
                                             lockm.CLUSTER_SCOPE)

    @mock.patch.object(common_utils, 'is_engine_dead')
    @mock.patch.object(svco.Service, 'gc_by_engine')
    @mock.patch.object(clo.ClusterLock, "acquire")
    @mock.patch.object(clo.ClusterLock, "steal")
    def test_cluster_lock_acquire_dead_owner(self, mock_steal, mock_acquire,
                                             mock_gc, mock_dead):
        mock_dead.return_value = True
        mock_acquire.return_value = ['ACTION_ABC']
        mock_steal.return_value = ['ACTION_XYZ']

        res = lockm.cluster_lock_acquire(self.ctx, 'CLUSTER_A', 'ACTION_XYZ',
                                         'NEW_ENGINE')

        self.assertTrue(res)
        mock_acquire.assert_called_with("CLUSTER_A", "ACTION_XYZ",
                                        lockm.CLUSTER_SCOPE)
        self.assertEqual(3, mock_acquire.call_count)
        mock_steal.assert_called_once_with('CLUSTER_A', 'ACTION_XYZ')
        mock_gc.assert_called_once_with(mock.ANY)

    @mock.patch.object(common_utils, 'is_engine_dead')
    @mock.patch.object(clo.ClusterLock, "acquire")
    def test_cluster_lock_acquire_failed(self, mock_acquire, mock_dead):
        mock_dead.return_value = False
        mock_acquire.return_value = ['ACTION_ABC']

        res = lockm.cluster_lock_acquire(self.ctx, 'CLUSTER_A', 'ACTION_XYZ')

        self.assertFalse(res)
        mock_acquire.assert_called_with('CLUSTER_A', 'ACTION_XYZ',
                                        lockm.CLUSTER_SCOPE)
        self.assertEqual(3, mock_acquire.call_count)

    @mock.patch.object(clo.ClusterLock, "acquire")
    @mock.patch.object(clo.ClusterLock, "steal")
    def test_cluster_lock_acquire_forced(self, mock_steal, mock_acquire):
        mock_acquire.return_value = ['ACTION_ABC']
        mock_steal.return_value = ['ACTION_XY']

        res = lockm.cluster_lock_acquire(self.ctx, 'CLUSTER_A',
                                         'ACTION_XY', forced=True)

        self.assertTrue(res)
        mock_acquire.assert_called_with('CLUSTER_A', 'ACTION_XY',
                                        lockm.CLUSTER_SCOPE)
        self.assertEqual(3, mock_acquire.call_count)
        mock_steal.assert_called_once_with('CLUSTER_A', 'ACTION_XY')

    @mock.patch.object(common_utils, 'is_engine_dead')
    @mock.patch.object(clo.ClusterLock, "acquire")
    @mock.patch.object(clo.ClusterLock, "steal")
    def test_cluster_lock_acquire_steal_failed(self, mock_steal, mock_acquire,
                                               mock_dead):
        mock_dead.return_value = False
        mock_acquire.return_value = ['ACTION_ABC']
        mock_steal.return_value = []

        res = lockm.cluster_lock_acquire(self.ctx, 'CLUSTER_A',
                                         'ACTION_XY', forced=True)

        self.assertFalse(res)
        mock_acquire.assert_called_with('CLUSTER_A', 'ACTION_XY',
                                        lockm.CLUSTER_SCOPE)
        self.assertEqual(3, mock_acquire.call_count)
        mock_steal.assert_called_once_with('CLUSTER_A', 'ACTION_XY')

    @mock.patch.object(clo.ClusterLock, "release")
    def test_cluster_lock_release(self, mock_release):
        actual = lockm.cluster_lock_release('C', 'A', 'S')

        self.assertEqual(mock_release.return_value, actual)
        mock_release.assert_called_once_with('C', 'A', 'S')

    @mock.patch.object(nlo.NodeLock, "acquire")
    def test_node_lock_acquire_already_owner(self, mock_acquire):
        mock_acquire.return_value = 'ACTION_XYZ'

        res = lockm.node_lock_acquire(self.ctx, 'NODE_A', 'ACTION_XYZ')

        self.assertTrue(res)
        mock_acquire.assert_called_once_with('NODE_A', 'ACTION_XYZ')

    @mock.patch.object(common_utils, 'is_engine_dead')
    @mock.patch.object(ao.Action, 'mark_failed')
    @mock.patch.object(nlo.NodeLock, "acquire")
    @mock.patch.object(nlo.NodeLock, "steal")
    def test_node_lock_acquire_dead_owner(self, mock_steal, mock_acquire,
                                          mock_action_fail, mock_dead):
        mock_dead.return_value = True
        mock_acquire.side_effect = ['ACTION_ABC']
        mock_steal.return_value = 'ACTION_XYZ'

        res = lockm.node_lock_acquire(self.ctx, 'NODE_A', 'ACTION_XYZ',
                                      'NEW_ENGINE')

        self.assertTrue(res)
        mock_acquire.assert_called_once_with('NODE_A', 'ACTION_XYZ')
        mock_steal.assert_called_once_with('NODE_A', 'ACTION_XYZ')
        mock_action_fail.assert_called_once_with(
            self.ctx, 'ACTION_ABC', mock.ANY,
            'Engine died when executing this action.')

    @mock.patch.object(common_utils, 'is_engine_dead')
    @mock.patch.object(nlo.NodeLock, "acquire")
    def test_node_lock_acquire_failed(self, mock_acquire, mock_dead):
        mock_dead.return_value = False
        mock_acquire.side_effect = ['ACTION_ABC']

        res = lockm.node_lock_acquire(self.ctx, 'NODE_A', 'ACTION_XYZ')

        self.assertFalse(res)
        mock_acquire.assert_called_once_with('NODE_A', 'ACTION_XYZ')

    @mock.patch.object(nlo.NodeLock, "acquire")
    @mock.patch.object(nlo.NodeLock, "steal")
    def test_node_lock_acquire_forced(self, mock_steal, mock_acquire):
        mock_acquire.side_effect = ['ACTION_ABC', 'ACTION_ABC', 'ACTION_ABC']
        mock_steal.return_value = 'ACTION_XY'

        res = lockm.node_lock_acquire(self.ctx, 'NODE_A',
                                      'ACTION_XY', forced=True)

        self.assertTrue(res)
        mock_acquire.assert_called_once_with('NODE_A', 'ACTION_XY')
        mock_steal.assert_called_once_with('NODE_A', 'ACTION_XY')

    @mock.patch.object(ao.Action, 'get')
    @mock.patch.object(nlo.NodeLock, "acquire")
    @mock.patch.object(nlo.NodeLock, "steal")
    def test_node_lock_acquire_steal_failed(self, mock_steal, mock_acquire,
                                            mock_get):
        mock_acquire.side_effect = ['ACTION_ABC']
        mock_steal.return_value = None

        res = lockm.node_lock_acquire(self.ctx, 'NODE_A',
                                      'ACTION_XY', forced=True)

        self.assertFalse(res)
        mock_acquire.assert_called_once_with('NODE_A', 'ACTION_XY')
        mock_steal.assert_called_once_with('NODE_A', 'ACTION_XY')

    @mock.patch.object(nlo.NodeLock, "release")
    def test_node_lock_release(self, mock_release):
        actual = lockm.node_lock_release('C', 'A')
        self.assertEqual(mock_release.return_value, actual)
        mock_release.assert_called_once_with('C', 'A')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
import six

from senlin.common import consts
from senlin.common import exception as exc
from senlin.common.i18n import _
from senlin.common import schema
from senlin.common import utils
from senlin.profiles import base

LOG = logging.getLogger(__name__)


class StackProfile(base.Profile):
    """Profile for an OpenStack Heat stack."""

    VERSIONS = {
        '1.0': [
            {'status': consts.SUPPORTED, 'since': '2016.04'}
        ]
    }

    KEYS = (
        CONTEXT, TEMPLATE, TEMPLATE_URL, PARAMETERS,
        FILES, TIMEOUT, DISABLE_ROLLBACK, ENVIRONMENT,
    ) = (
        'context', 'template', 'template_url', 'parameters',
        'files', 'timeout', 'disable_rollback', 'environment',
    )

    properties_schema = {
        CONTEXT: schema.Map(
            _('A dictionary for specifying the customized context for '
              'stack operations'),
            default={},
        ),
        TEMPLATE: schema.Map(
            _('Heat stack template.'),
            default={},
            updatable=True,
        ),
        TEMPLATE_URL: schema.String(
            _('Heat stack template url.'),
            default='',
            updatable=True,
        ),
        PARAMETERS: schema.Map(
            _('Parameters to be passed to Heat for stack operations.'),
            default={},
            updatable=True,
        ),
        FILES: schema.Map(
            _('Contents of files referenced by the template, if any.'),
            default={},
            updatable=True,
        ),
        TIMEOUT: schema.Integer(
            _('A integer that specifies the number of minutes that a '
              'stack operation times out.'),
            updatable=True,
        ),
        DISABLE_ROLLBACK: schema.Boolean(
            _('A boolean specifying whether a stack operation can be '
              'rolled back.'),
            default=True,
            updatable=True,
        ),
        ENVIRONMENT: schema.Map(
            _('A map that specifies the environment used for stack '
              'operations.'),
            default={},
            updatable=True,
        )
    }

    OP_NAMES = (
        OP_ABANDON,
    ) = (
        'abandon',
    )

    OPERATIONS = {
        OP_ABANDON: schema.Map(
            _('Abandon a heat stack node.'),
        )
    }

    def __init__(self, type_name, name, **kwargs):
        super(StackProfile, self).__init__(type_name, name, **kwargs)
        self.stack_id = None

    def validate(self, validate_props=False):
        """Validate the schema and the data provided."""
        # general validation
        self.spec_data.validate()
        self.properties.validate()
        # validate template
        template = self.properties[self.TEMPLATE]
        template_url = self.properties[self.TEMPLATE_URL]
        if not template and not template_url:
            msg = _("Both template and template_url are not specified "
                    "for profile '%s'.") % self.name
            raise exc.InvalidSpec(message=msg)

        if validate_props:
            self.do_validate(obj=self)

    def do_validate(self, obj):
        """Validate the stack template used by a node.

        :param obj: Node object to operate.
        :returns: True if validation succeeds.
        :raises: `InvalidSpec` exception is raised if template is invalid.
        """
        kwargs = {
            'stack_name': utils.random_name(),
            'template': self.properties[self.TEMPLATE],
            'template_url': self.properties[self.TEMPLATE_URL],
            'parameters': self.properties[self.PARAMETERS],
            'files': self.properties[self.FILES],
            'environment': self.properties[self.ENVIRONMENT],
            'preview': True,
        }
        try:
            self.orchestration(obj).stack_create(**kwargs)
        except exc.InternalError as ex:
            msg = _('Failed in validating template: %s') % six.text_type(ex)
            raise exc.InvalidSpec(message=msg)

        return True

    def do_create(self, obj):
        """Create a heat stack using the given node object.

        :param obj: The node object to operate on.
        :returns: The UUID of the heat stack created.
        """
        tags = ["cluster_node_id=%s" % obj.id]
        if obj.cluster_id:
            tags.append('cluster_id=%s' % obj.cluster_id)
            tags.append('cluster_node_index=%s' % obj.index)
        kwargs = {
            'stack_name': obj.name + '-' + utils.random_name(8),
            'template': self.properties[self.TEMPLATE],
            'template_url': self.properties[self.TEMPLATE_URL],
            'timeout_mins': self.properties[self.TIMEOUT],
            'disable_rollback': self.properties[self.DISABLE_ROLLBACK],
            'parameters': self.properties[self.PARAMETERS],
            'files': self.properties[self.FILES],
            'environment': self.properties[self.ENVIRONMENT],
            'tags': ",".join(tags)
        }

        try:
            stack = self.orchestration(obj).stack_create(**kwargs)

            # Timeout = None means we will use the 'default_action_timeout'
            # It can be overridden by the TIMEOUT profile properties
            timeout = None
            if self.properties[self.TIMEOUT]:
                timeout = self.properties[self.TIMEOUT] * 60

            self.orchestration(obj).wait_for_stack(stack.id, 'CREATE_COMPLETE',
                                                   timeout=timeout)
            return stack.id
        except exc.InternalError as ex:
            raise exc.EResourceCreation(type='stack',
                                        message=six.text_type(ex))

    def do_delete(self, obj, **params):
        """Delete the physical stack behind the node object.

        :param obj: The node object to operate on.
        :param kwargs params: Optional keyword arguments for the delete
                              operation.
        :returns: This operation always returns True unless exception is
                  caught.
        :raises: `EResourceDeletion` if interaction with heat fails.
        """
        stack_id = obj.physical_id
        if not stack_id:
            return True

        ignore_missing = params.get('ignore_missing', True)
        try:
            self.orchestration(obj).stack_delete(stack_id, ignore_missing)
            self.orchestration(obj).wait_for_stack_delete(stack_id)
        except exc.InternalError as ex:
            raise exc.EResourceDeletion(type='stack', id=stack_id,
                                        message=six.text_type(ex))
        return True

    def do_update(self, obj, new_profile, **params):
        """Perform update on object.

        :param obj: the node object to operate on
        :param new_profile: the new profile used for updating
        :param params: other parameters for the update request.
        :returns: A boolean indicating whether the operation is successful.
        """
        self.stack_id = obj.physical_id
        if not self.stack_id:
            return False

        if not self.validate_for_update(new_profile):
            return False

        fields = {}
        new_template = new_profile.properties[new_profile.TEMPLATE]
        if new_template != self.properties[self.TEMPLATE]:
            fields['template'] = new_template

        new_params = new_profile.properties[new_profile.PARAMETERS]
        if new_params != self.properties[self.PARAMETERS]:
            fields['parameters'] = new_params

        new_timeout = new_profile.properties[new_profile.TIMEOUT]
        if new_timeout != self.properties[self.TIMEOUT]:
            fields['timeout_mins'] = new_timeout

        new_dr = new_profile.properties[new_profile.DISABLE_ROLLBACK]
        if new_dr != self.properties[self.DISABLE_ROLLBACK]:
            fields['disable_rollback'] = new_dr

        new_files = new_profile.properties[new_profile.FILES]
        if new_files != self.properties[self.FILES]:
            fields['files'] = new_files

        new_environment = new_profile.properties[new_profile.ENVIRONMENT]
        if new_environment != self.properties[self.ENVIRONMENT]:
            fields['environment'] = new_environment

        if not fields:
            return True

        try:
            hc = self.orchestration(obj)
            # Timeout = None means we will use the 'default_action_timeout'
            # It can be overridden by the TIMEOUT profile properties
            timeout = None
            if self.properties[self.TIMEOUT]:
                timeout = self.properties[self.TIMEOUT] * 60
            hc.stack_update(self.stack_id, **fields)
            hc.wait_for_stack(self.stack_id, 'UPDATE_COMPLETE',
                              timeout=timeout)
        except exc.InternalError as ex:
            raise exc.EResourceUpdate(type='stack', id=self.stack_id,
                                      message=six.text_type(ex))

        return True

    def do_check(self, obj):
        """Check stack status.

        :param obj: Node object to operate.
        :returns: True if check succeeded, or False otherwise.
        """
        stack_id = obj.physical_id
        if stack_id is None:
            return False

        hc = self.orchestration(obj)
        try:
            # Timeout = None means we will use the 'default_action_timeout'
            # It can be overridden by the TIMEOUT profile properties
            timeout = None
            if self.properties[self.TIMEOUT]:
                timeout = self.properties[self.TIMEOUT] * 60
            hc.stack_check(stack_id)
            hc.wait_for_stack(stack_id, 'CHECK_COMPLETE', timeout=timeout)
        except exc.InternalError as ex:
            raise exc.EResourceOperation(op='checking', type='stack',
                                         id=stack_id,
                                         message=six.text_type(ex))

        return True

    def do_get_details(self, obj):
        if not obj.physical_id:
            return {}

        try:
            stack = self.orchestration(obj).stack_get(obj.physical_id)
            return stack.to_dict()
        except exc.InternalError as ex:
            return {
                'Error': {
                    'code': ex.code,
                    'message': six.text_type(ex)
                }
            }

    def do_adopt(self, obj, overrides=None, snapshot=False):
        """Adopt an existing stack node for management.

        :param obj: A node object for this operation. It could be a puppet
            node that provides only 'user', 'project' and 'physical_id'
            properties when doing a preview. It can be a real Node object for
            node adoption.
        :param overrides: A dict containing the properties that will be
            overridden when generating a profile for the stack.
        :param snapshot: A boolean flag indicating whether the profile should
            attempt a snapshot operation before adopting the stack. If set to
            True, the ID of the snapshot will be used as the image ID.

        :returns: A dict containing the spec created from the stack object or
            a dict containing error information if failure occurred.
        """
        driver = self.orchestration(obj)

        # TODO(Qiming): Add snapshot support
        # snapshot = driver.snapshot_create(...)

        try:
            stack = driver.stack_get(obj.physical_id)
            tmpl = driver.stack_get_template(obj.physical_id)
            env = driver.stack_get_environment(obj.physical_id)
            files = driver.stack_get_files(obj.physical_id)
        except exc.InternalError as ex:
            return {'Error': {'code': ex.code, 'message': six.text_type(ex)}}

        spec = {
            self.ENVIRONMENT: env.to_dict(),
            self.FILES: files,
            self.TEMPLATE: tmpl.to_dict(),
            self.PARAMETERS: dict((k, v) for k, v in stack.parameters.items()
                                  if k.find('OS::', 0) < 0),
            self.TIMEOUT: stack.timeout_mins,
            self.DISABLE_ROLLBACK: stack.is_rollback_disabled
        }
        if overrides:
            spec.update(overrides)

        return spec

    def _refresh_tags(self, current, node, add=False):
        """Refresh tag list.

        :param current: Current list of tags.
        :param node: The node object.
        :param add: Flag indicating whether new tags are added.
        :returns: (tags, updated) where tags contains a new list of tags and
                  updated indicates whether new tag list differs from the old
                  one.
        """
        tags = []
        for tag in current:
            if tag.find('cluster_id=') == 0:
                continue
            elif tag.find('cluster_node_id=') == 0:
                continue
            elif tag.find('cluster_node_index=') == 0:
                continue
            if tag.strip() != "":
                tags.append(tag.strip())

        if add:
            tags.append('cluster_id=' + node.cluster_id)
            tags.append('cluster_node_id=' + node.id)
            tags.append('cluster_node_index=%s' % node.index)

        tag_str = ",".join(tags)
        return (tag_str, tags != current)

    def do_join(self, obj, cluster_id):
        if not obj.physical_id:
            return False

        hc = self.orchestration(obj)
        try:
            stack = hc.stack_get(obj.physical_id)
            tags, updated = self._refresh_tags(stack.tags, obj, True)
            field = {'tags': tags}
            if updated:
                hc.stack_update(obj.physical_id, **field)
        except exc.InternalError as ex:
            LOG.error('Failed in updating stack tags: %s.', ex)
            return False

        return True

    def do_leave(self, obj):
        if not obj.physical_id:
            return False

        hc = self.orchestration(obj)
        try:
            stack = hc.stack_get(obj.physical_id)
            tags, updated = self._refresh_tags(stack.tags, obj, False)
            field = {'tags': tags}
            if updated:
                hc.stack_update(obj.physical_id, **field)
        except exc.InternalError as ex:
            LOG.error('Failed in updating stack tags: %s.', ex)
            return False

        return True

    def handle_abandon(self, obj, **options):
        """Handler for abandoning a heat stack node."""
        pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from oslo_config import cfg
import six
import webob

from senlin.api.common import util as common_util
from senlin.api.common import version_request as vr
from senlin.api.middleware import webhook as webhook_middleware
from senlin.common import context
from senlin.common import exception
from senlin.drivers import base as driver_base
from senlin.rpc import client as rpc
from senlin.tests.unit.common import base
from senlin.tests.unit.common import utils


class TestWebhookMiddleware(base.SenlinTestCase):

    def setUp(self):
        super(TestWebhookMiddleware, self).setUp()
        self.ctx = utils.dummy_context()
        self.middleware = webhook_middleware.WebhookMiddleware(None)

        self.url_slices = {
            '00_url_base': 'http://HOST_IP:PORT/v1',
            '01_webhook_str': '/webhooks/',
            '02_webhook_id': 'WEBHOOK_ID',
            '03_trigger_str': '/trigger?',
            '04_version': 'V=1',
            '05_params': '&key=TEST_KEY',
        }
        self.credential = {
            'auth_url': 'TEST_URL',
            'user_id': '123',
            'password': 'abc'
        }

    def _generate_url(self):
        slices = sorted(self.url_slices.items(), key=lambda d: d[0])
        return ''.join(s[1] for s in slices)

    def test_parse_url(self):
        # Get webhook_id correctly
        res = self.middleware._parse_url(self._generate_url())
        self.assertEqual(('WEBHOOK_ID', {'key': 'TEST_KEY'}), res)

    def test_parse_url_version_provided_no_key(self):
        # The structure /<webhook_id>/trigger?V=1 should be valid
        self.url_slices.pop('05_params')

        res = self.middleware._parse_url(self._generate_url())
        self.assertEqual(('WEBHOOK_ID', {}), res)

    def test_parse_url_no_version_provided_no_key_provided(self):
        # The structure /<webhook_id>/trigger should be invalid
        # because version is missing
        self.url_slices.pop('04_version')
        self.url_slices.pop('05_params')

        ex = self.assertRaises(webob.exc.HTTPBadRequest,
                               self.middleware._parse_url,
                               self._generate_url())

        self.assertEqual("V query parameter is required in webhook trigger "
                         "URL", six.text_type(ex))

    def test_parse_url_no_version_provided_key_provided(self):
        # The structure /<webhook_id>/trigger?key=value should be invalid
        # because version is missing
        self.url_slices.pop('04_version')
        self.url_slices.pop('05_params')
        self.url_slices['05_params'] = 'key=TEST_KEY'

        ex = self.assertRaises(webob.exc.HTTPBadRequest,
                               self.middleware._parse_url,
                               self._generate_url())

        self.assertEqual("V query parameter is required in webhook trigger "
                         "URL", six.text_type(ex))

    def test_parse_url_webhooks_not_found(self):
        # String 'webhooks' is not found in url
        self.url_slices['01_webhook_str'] = '/foo/'
        res = self.middleware._parse_url(self._generate_url())
        self.assertIsNone(res)

    def test_parse_url_trigger_not_found(self):
        # String 'trigger' is not found in url
        self.url_slices['03_trigger_str'] = '/foo?'
        res = self.middleware._parse_url(self._generate_url())
        self.assertIsNone(res)

    def test_parse_url_illegal_format(self):
        # The structure /<webhook_id>/trigger?key=value is not matched
        self.url_slices['03_trigger_str'] = 'trigger?'
        res = self.middleware._parse_url(self._generate_url())
        self.assertIsNone(res)

    def test_parse_url_no_trigger_word(self):
        # Bottom string of the url does not start with 'trigger'
        self.url_slices['03_trigger_str'] = '/foo-trigger?'
        res = self.middleware._parse_url(self._generate_url())
        self.assertIsNone(res)

    @mock.patch.object(driver_base, 'SenlinDriver')
    def test_get_token_succeeded(self, mock_senlindriver):
        class FakeAccessInfo(object):
            def __init__(self, auth_token):
                self.auth_token = auth_token

        sd = mock.Mock()
        sd.identity.get_token.return_value = 'TEST_TOKEN'
        mock_senlindriver.return_value = sd

        token = self.middleware._get_token(**self.credential)
        self.assertEqual('TEST_TOKEN', token)

    @mock.patch.object(driver_base, 'SenlinDriver')
    def test_get_token_failed(self, mock_senlindriver):
        self.credential['webhook_id'] = 'WEBHOOK_ID'

        sd = mock.Mock()
        sd.identity.get_token.side_effect = Exception()
        mock_senlindriver.return_value = sd

        self.assertRaises(exception.Forbidden, self.middleware._get_token,
                          **self.credential)

    @mock.patch.object(common_util, 'parse_request')
    @mock.patch.object(context, 'RequestContext')
    @mock.patch.object(rpc, 'EngineClient')
    def test_process_request(self, mock_client, mock_ctx, mock_parse):
        cfg.CONF.set_override('auth_url', 'AUTH_URL', group='authentication')
        cfg.CONF.set_override('service_username', 'USERNAME',
                              group='authentication')
        cfg.CONF.set_override('service_user_domain', 'DOMAIN',
                              group='authentication')
        cfg.CONF.set_override('service_password', 'PASSWORD',
                              group='authentication')

        req = mock.Mock()
        req.method = 'POST'
        req.url = 'http://url1'
        req.params = {'key': 'FAKE_KEY'}
        req.headers = {}
        req.version_request = vr.APIVersionRequest('1.0')

        rpcc = mock.Mock()
        fake_receiver = {
            'id': 'FAKE_ID',
            'actor': {'foo': 'bar'}
        }
        rpcc.call.return_value = fake_receiver
        mock_client.return_value = rpcc
        dbctx = mock.Mock()
        mock_ctx.return_value = dbctx

        obj = mock.Mock()
        mock_parse.return_value = obj

        fake_return = ('WEBHOOK', {})
        mock_extract = self.patchobject(self.middleware, '_parse_url',
                                        return_value=fake_return)
        mock_token = self.patchobject(self.middleware, '_get_token',
                                      return_value='FAKE_TOKEN')

        res = self.middleware.process_request(req)
        self.assertIsNone(res)

        self.assertEqual('FAKE_TOKEN', req.headers['X-Auth-Token'])
        mock_extract.assert_called_once_with('http://url1')
        mock_token.assert_called_once_with(
            auth_url='AUTH_URL', password='PASSWORD', username='USERNAME',
            user_domain_name='DOMAIN', foo='bar')

        mock_parse.assert_called_once_with('ReceiverGetRequest', req,
                                           {'identity': 'WEBHOOK'})
        rpcc.call.assert_called_with(dbctx, 'receiver_get', obj)

    def test_process_request_method_not_post(self):
        # Request method is not POST
        req = mock.Mock()
        req.method = 'GET'
        res = self.middleware.process_request(req)
        self.assertIsNone(res)
        self.assertNotIn('X-Auth-Token', req.headers)

    def test_process_request_bad_format(self):
        # no webhook_id extracted
        req = mock.Mock()
        req.method = 'POST'
        req.url = 'http://url1'
        mock_extract = self.patchobject(self.middleware, '_parse_url',
                                        return_value=None)

        res = self.middleware.process_request(req)
        self.assertIsNone(res)
        mock_extract.assert_called_once_with(req.url)
        self.assertNotIn('X-Auth-Token', req.headers)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import docker

from senlin.drivers import sdk


class DockerClient(object):
    """Docker driver."""

    def __init__(self, url):
        self._dockerclient = docker.APIClient(base_url=url, version='auto')

    @sdk.translate_exception
    def container_create(self, image, name=None, command=None):
        return self._dockerclient.create_container(name=name, image=image,
                                                   command=command)

    @sdk.translate_exception
    def container_delete(self, container):
        self._dockerclient.remove_container(container)
        return True

    @sdk.translate_exception
    def restart(self, container, timeout=None):
        params = {'timeout': timeout} if timeout else {}
        self._dockerclient.restart(container, **params)

    @sdk.translate_exception
    def pause(self, container):
        self._dockerclient.pause(container)

    @sdk.translate_exception
    def unpause(self, container):
        self._dockerclient.unpause(container)

    @sdk.translate_exception
    def start(self, container):
        self._dockerclient.start(container)

    @sdk.translate_exception
    def stop(self, container, timeout=None):
        params = {'timeout': timeout}
        self._dockerclient.stop(container, **params)

    @sdk.translate_exception
    def rename(self, container, name):
        self._dockerclient.rename(container, name)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Cluster endpoint for Senlin v1 REST API.
"""

from webob import exc

from senlin.api.common import util
from senlin.api.common import wsgi
from senlin.common import consts
from senlin.common.i18n import _


class ClusterController(wsgi.Controller):
    """WSGI controller for clusters resource in Senlin v1 API."""

    # Define request scope
    # (must match what is in policy file and policies in code.)
    REQUEST_SCOPE = 'clusters'

    SUPPORTED_ACTIONS = (
        ADD_NODES, DEL_NODES, SCALE_OUT, SCALE_IN, RESIZE,
        POLICY_ATTACH, POLICY_DETACH, POLICY_UPDATE,
        CHECK, RECOVER, REPLACE_NODES, COMPLETE_LIFECYCLE
    ) = (
        'add_nodes', 'del_nodes', 'scale_out', 'scale_in', 'resize',
        'policy_attach', 'policy_detach', 'policy_update',
        'check', 'recover', 'replace_nodes', 'complete_lifecycle'
    )

    @util.policy_enforce
    def index(self, req):
        whitelist = {
            consts.CLUSTER_NAME: 'mixed',
            consts.CLUSTER_STATUS: 'mixed',
            consts.PARAM_LIMIT: 'single',
            consts.PARAM_MARKER: 'single',
            consts.PARAM_SORT: 'single',
            consts.PARAM_GLOBAL_PROJECT: 'single',
        }
        for key in req.params.keys():
            if key not in whitelist:
                raise exc.HTTPBadRequest(_("Invalid parameter '%s'") % key)

        params = util.get_allowed_params(req.params, whitelist)
        # Note: We have to do a boolean parsing here because 1) there is
        # a renaming, 2) the boolean is usually presented as a string.
        is_global = params.pop(consts.PARAM_GLOBAL_PROJECT, False)
        unsafe = util.parse_bool_param(consts.PARAM_GLOBAL_PROJECT, is_global)
        params['project_safe'] = not unsafe
        req_obj = util.parse_request('ClusterListRequest', req, params)
        clusters = self.rpc_client.call(req.context, 'cluster_list', req_obj)
        return {'clusters': clusters}

    @util.policy_enforce
    def create(self, req, body):
        """Create a new cluster."""
        obj = util.parse_request('ClusterCreateRequest', req, body, 'cluster')
        cluster = self.rpc_client.call(req.context, 'cluster_create',
                                       obj.cluster)
        action_id = cluster.pop('action')
        result = {
            'cluster': cluster,
            'location': '/actions/%s' % action_id,
        }
        return result

    @util.policy_enforce
    def get(self, req, cluster_id):
        """Gets detailed information for a cluster."""
        body = {'identity': cluster_id}
        obj = util.parse_request('ClusterGetRequest', req, body)
        cluster = self.rpc_client.call(req.context, 'cluster_get', obj)

        return {'cluster': cluster}

    @util.policy_enforce
    def update(self, req, cluster_id, body):
        """Update an existing cluster with new parameters."""
        data = body.get('cluster')
        if data is None:
            raise exc.HTTPBadRequest(_("Malformed request data, missing "
                                       "'cluster' key in request body."))
        params = body['cluster']
        params['identity'] = cluster_id

        obj = util.parse_request('ClusterUpdateRequest', req, params)
        cluster = self.rpc_client.call(req.context, 'cluster_update', obj)

        action_id = cluster.pop('action')
        result = {
            'cluster': cluster,
            'location': '/actions/%s' % action_id,
        }
        return result

    def _do_add_nodes(self, req, cid, data):
        nodes = data.get('nodes', [])
        params = {'identity': cid, 'nodes': nodes}
        obj = util.parse_request('ClusterAddNodesRequest', req, params)
        return self.rpc_client.call(req.context, 'cluster_add_nodes', obj)

    def _do_del_nodes(self, req, cid, data):
        nodes = data.get('nodes', [])
        destroy = data.get('destroy_after_deletion', False)
        params = {'identity': cid, 'nodes': nodes,
                  'destroy_after_deletion': destroy}
        obj = util.parse_request('ClusterDelNodesRequest', req, params)
        return self.rpc_client.call(req.context, 'cluster_del_nodes', obj)

    @wsgi.Controller.api_version('1.3')
    def _do_replace_nodes(self, req, cluster_id, data):
        nodes = data.get('nodes', {})
        if not nodes or not isinstance(nodes, dict):
            msg = _("The data provided is not a map")
            raise exc.HTTPBadRequest(msg)

        params = {'identity': cluster_id, 'nodes': nodes}
        obj = util.parse_request('ClusterReplaceNodesRequest', req, params)
        return self.rpc_client.call(req.context, 'cluster_replace_nodes',
                                    obj)

    def _do_resize(self, req, cluster_id, data):
        params = {}

        for key in [consts.ADJUSTMENT_TYPE, consts.ADJUSTMENT_NUMBER,
                    consts.ADJUSTMENT_MIN_SIZE, consts.ADJUSTMENT_MAX_SIZE]:
            if data.get(key, None) is not None:
                params[key] = data.get(key)

        adj_type = data.get(consts.ADJUSTMENT_TYPE, None)
        min_step = data.get(consts.ADJUSTMENT_MIN_STEP, None)
        if ((adj_type == consts.CHANGE_IN_PERCENTAGE) and
                min_step is not None):
            params[consts.ADJUSTMENT_MIN_STEP] = min_step

        if not params:
            msg = _("Not enough parameters to do resize action.")
            raise exc.HTTPBadRequest(msg)

        strict = data.get(consts.ADJUSTMENT_STRICT, None)
        if strict is not None:
            params[consts.ADJUSTMENT_STRICT] = strict

        params['identity'] = cluster_id
        obj = util.parse_request('ClusterResizeRequest', req, params)

        if (obj.obj_attr_is_set('adjustment_type') and
                not obj.obj_attr_is_set('number')):
            msg = _("Missing number value for size adjustment.")
            raise exc.HTTPBadRequest(msg)

        if (obj.obj_attr_is_set('number') and
                not obj.obj_attr_is_set('adjustment_type')):
            msg = _("Missing adjustment_type value for size adjustment.")
            raise exc.HTTPBadRequest(msg)

        if (obj.obj_attr_is_set('min_size') and
                obj.obj_attr_is_set('max_size')):
            if obj.max_size > 0 and obj.min_size > obj.max_size:
                msg = _("The specified min_size (%(n)s) is greater than the "
                        "specified max_size (%(m)s)."
                        ) % {'m': obj.max_size, 'n': obj.min_size}
                raise exc.HTTPBadRequest(msg)

        return self.rpc_client.call(req.context, 'cluster_resize', obj)

    def _do_scale_out(self, req, cid, data):
        count = data.get('count', None)
        params = {'identity': cid}
        if count is not None:
            params['count'] = count
        obj = util.parse_request('ClusterScaleOutRequest', req, params)
        return self.rpc_client.call(req.context, 'cluster_scale_out', obj)

    def _do_scale_in(self, req, cid, data):
        count = data.get('count', None)
        params = {'identity': cid}

        if count is not None:
            params['count'] = count

        obj = util.parse_request('ClusterScaleInRequest', req, params)
        return self.rpc_client.call(req.context, 'cluster_scale_in', obj)

    def _do_policy_attach(self, req, cid, data):
        params = {'identity': cid}
        params.update(data)
        obj = util.parse_request('ClusterAttachPolicyRequest', req, params)
        return self.rpc_client.call(req.context,
                                    'cluster_policy_attach', obj)

    def _do_policy_detach(self, req, cid, data):
        params = {'identity': cid}
        params.update(data)

        obj = util.parse_request('ClusterDetachPolicyRequest', req, params)
        return self.rpc_client.call(req.context,
                                    'cluster_policy_detach', obj)

    def _do_policy_update(self, req, cid, data):
        params = {'identity': cid}
        params.update(data)

        obj = util.parse_request('ClusterUpdatePolicyRequest', req, params)
        return self.rpc_client.call(req.context,
                                    'cluster_policy_update', obj)

    def _do_check(self, req, cid, data):
        params = {'identity': cid, 'params': data}
        obj = util.parse_request('ClusterCheckRequest', req, params)
        return self.rpc_client.call(req.context, 'cluster_check', obj)

    def _do_recover(self, req, cid, data):
        params = {'identity': cid, 'params': data}
        obj = util.parse_request('ClusterRecoverRequest', req, params)
        return self.rpc_client.call(req.context, 'cluster_recover', obj)

    @wsgi.Controller.api_version('1.9')
    def _do_complete_lifecycle(self, req, cid, data):
        lifecycle_action_token = data.get('lifecycle_action_token', None)

        params = {'identity': cid,
                  'lifecycle_action_token': lifecycle_action_token}
        obj = util.parse_request('ClusterCompleteLifecycleRequest', req,
                                 params)
        return self.rpc_client.call(req.context, 'cluster_complete_lifecycle',
                                    obj)

    @util.policy_enforce
    def action(self, req, cluster_id, body=None):
        """Perform specified action on a cluster."""
        body = body or {}
        if len(body) < 1:
            raise exc.HTTPBadRequest(_('No action specified'))

        if len(body) > 1:
            raise exc.HTTPBadRequest(_('Multiple actions specified'))

        this_action = list(body.keys())[0]
        if this_action not in self.SUPPORTED_ACTIONS:
            msg = _("Unrecognized action '%s' specified") % this_action
            raise exc.HTTPBadRequest(msg)

        do_func_name = "_do_" + this_action
        if not hasattr(self, do_func_name):
            raise exc.HTTPBadRequest(_('Unsupported action'))

        do_func = getattr(self, do_func_name)
        data = body.get(this_action, {})
        if not isinstance(data, dict):
            msg = _("The data provided is not a map")
            raise exc.HTTPBadRequest(msg)

        res = do_func(req, cluster_id, data)

        location = {'location': '/actions/%s' % res['action']}
        res.update(location)
        return res

    @wsgi.Controller.api_version('1.2')
    @util.policy_enforce
    def collect(self, req, cluster_id, path):
        """Aggregate attribute values across a cluster."""
        stripped_path = path.strip()
        if (stripped_path == '') or (stripped_path == 'None'):
            raise exc.HTTPBadRequest(_("Required path attribute is missing."))

        params = {
            'identity': cluster_id,
            'path': stripped_path,
        }
        obj = util.parse_request('ClusterCollectRequest', req, params)
        return self.rpc_client.call(req.context, 'cluster_collect', obj)

    @wsgi.Controller.api_version('1.4')
    @util.policy_enforce
    def operation(self, req, cluster_id, body=None):
        """Perform specified operation on the specified cluster."""
        body = body or {}
        if len(body) < 1:
            raise exc.HTTPBadRequest(_('No operation specified'))

        if len(body) > 1:
            raise exc.HTTPBadRequest(_('Multiple operations specified'))

        operation = list(body.keys())[0]
        params = {
            'identity': cluster_id,
            'operation': operation,
            'params': body[operation].get('params', {}),
            'filters': body[operation].get('filters', {}),
        }
        obj = util.parse_request('ClusterOperationRequest', req, params)

        res = self.rpc_client.call(req.context, 'cluster_op', obj)

        location = {'location': '/actions/%s' % res['action']}
        res.update(location)
        return res

    @util.policy_enforce
    def delete(self, req, cluster_id, body=None):
        if req.params.get('force') is not None:
            force = util.parse_bool_param(consts.CLUSTER_DELETE_FORCE,
                                          req.params.get('force'))
        elif body:
            force = body.get('force')
            if force is None:
                force = False
        else:
            force = False

        params = {'identity': cluster_id, 'force': force}
        obj = util.parse_request('ClusterDeleteRequest', req, params)
        res = self.rpc_client.call(req.context, 'cluster_delete', obj)

        action_id = res.pop('action')
        result = {'location': '/actions/%s' % action_id}
        return result



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
A module that contains various fake entities
"""

from senlin.common import schema
from senlin.policies import base as policy_base
from senlin.profiles import base as profile_base


class TestProfile(profile_base.Profile):

    CONTEXT = 'context'

    properties_schema = {
        CONTEXT: schema.Map("context property"),
        'INT': schema.Integer('int property', default=0),
        'STR': schema.String('string property', default='a string'),
        'MAP': schema.Map(
            'map property',
            schema={
                'KEY1': schema.Integer('key1'),
                'KEY2': schema.String('key2')
            }
        ),
        'LIST': schema.List(
            'list property',
            schema=schema.String('list item'),
        ),
    }

    OPERATIONS = {}

    def __init__(self, name, spec, **kwargs):
        super(TestProfile, self).__init__(name, spec, **kwargs)

    @classmethod
    def delete(cls, ctx, profile_id):
        super(TestProfile, cls).delete(ctx, profile_id)

    def do_create(self):
        return {}

    def do_delete(self, id):
        return True

    def do_update(self):
        return {}

    def do_check(self, id):
        return True


class TestPolicy(policy_base.Policy):
    VERSION = 1.0
    properties_schema = {
        'KEY1': schema.String('key1', default='default1'),
        'KEY2': schema.Integer('key2', required=True),
    }

    TARGET = [
        ('BEFORE', 'CLUSTER_ADD_NODES')
    ]

    def __init__(self, name, spec, **kwargs):
        super(TestPolicy, self).__init__(name, spec, **kwargs)

    def attach(self, cluster, enabled=True):
        return True, {}

    def detach(self, cluster):
        return True, 'OK'

    def pre_op(self, cluster_id, action):
        return

    def post_op(self, cluster_id, action):
        return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from senlin.objects.requests import policy_type
from senlin.tests.unit.common import base as test_base


class TestPolicyTypeGet(test_base.SenlinTestCase):

    def test_policy_type_get(self):
        sot = policy_type.PolicyTypeGetRequest(type_name='Fake')

        self.assertEqual('Fake', sot.type_name)

    def test_policy_type_to_primitive(self):
        sot = policy_type.PolicyTypeGetRequest(type_name='Fake')

        res = sot.obj_to_primitive()

        self.assertIn('type_name', res['senlin_object.changes'])
        self.assertEqual(u'Fake', res['senlin_object.data']['type_name'])
        self.assertEqual('PolicyTypeGetRequest', res['senlin_object.name'])
        self.assertEqual('senlin', res['senlin_object.namespace'])
        self.assertEqual('1.0', res['senlin_object.version'])


class TestPolicyTypeList(test_base.SenlinTestCase):

    def test_policy_type_list_to_primitive(self):
        sot = policy_type.PolicyTypeListRequest()
        res = sot.obj_to_primitive()

        self.assertEqual({}, res['senlin_object.data'])
        self.assertEqual('PolicyTypeListRequest', res['senlin_object.name'])
        self.assertEqual('senlin', res['senlin_object.namespace'])
        self.assertEqual('1.0', res['senlin_object.version'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from senlin.drivers import base
from senlin.drivers import sdk


class CinderClient(base.DriverBase):
    """Fake Cinder V2 driver for test."""
    def __init__(self, ctx):
        self.fake_volume_create = {
            "id": "3095aefc-09fb-4bc7-b1f0-f21a304e864c",
            "size": 2,
            "links": [
                {
                    "href": " ",
                    "rel": "self"
                }
            ]
        }

        self.fake_volume_get = {
            "status": "available",
            "attachments": [],
            "links": [
                {
                    "href": " ",
                    "rel": "self"
                },
                {
                    "href": " ",
                    "rel": "bookmark"
                }
            ],
            "availability_zone": "nova",
            "bootable": "false",
            "os-vol-host-attr:host": "ip-10-168-107-25",
            "source_volid": "",
            "snapshot_id": "",
            "id": "5aa119a8-d25b-45a7-8d1b-88e127885635",
            "description": "Super volume.",
            "name": "vol-002",
            "created_at": "2013-02-25T02:40:21.000000",
            "volume_type": "None",
            "os-vol-tenant-attr:tenant_id": "0c2eba2c5af04d3f9e9d0d410b371fde",
            "size": 1,
            "os-volume-replication:driver_data": "",
            "os-volume-replication:extended_status": "",
            "metadata": {
                "contents": "not junk"
            }
        }

        self.fake_snapshot_create = {
            "name": "snap-001",
            "description": "Daily backup",
            "volume_id": "5aa119a8-d25b-45a7-8d1b-88e127885635",
            "force": True
        }

        self.fake_snapshot_get = {
            "status": "available",
            "os-extended-snapshot-attributes:progress": "100%",
            "description": "Daily backup",
            "created_at": "2013-02-25T04:13:17.000000",
            "metadata": {},
            "volume_id": "5aa119a8-d25b-45a7-8d1b-88e127885635",
            "os-extended-snapshot-attributes:project_id":
                "0c2eba2c5af04d3f9e9d0d410b371fde",
            "size": 1,
            "id": "2bb856e1-b3d8-4432-a858-09e4ce939389",
            "name": "snap-001"
        }

    def volume_create(self, **params):
        return sdk.FakeResourceObject(self.fake_volume_create)

    def volume_get(self, volume_id):
        sdk.FakeResourceObject(self.fake_volume_get)

    def volume_delete(self, volume_id, ignore_missing=True):
        return

    def snapshot_create(self, **params):
        return sdk.FakeResourceObject(self.fake_snapshot_create)

    def snapshot_get(self, volume_id):
        sdk.FakeResourceObject(self.fake_snapshot_get)

    def snapshot_delete(self, volume_id, ignore_missing=True):
        return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import random
import string

from oslo_config import cfg
from oslo_db import options
from oslo_utils import timeutils
import sqlalchemy

from senlin.common import context
from senlin.db import api as db_api
from senlin import objects


def random_name():
    return ''.join(random.choice(string.ascii_uppercase)
                   for x in range(10))


def setup_dummy_db():
    options.cfg.set_defaults(options.database_opts, sqlite_synchronous=False)
    options.set_defaults(cfg.CONF, connection="sqlite://")
    engine = db_api.get_engine()
    db_api.db_sync(engine)
    engine.connect()


def reset_dummy_db():
    engine = db_api.get_engine()
    meta = sqlalchemy.MetaData()
    meta.reflect(bind=engine)

    for table in reversed(meta.sorted_tables):
        if table.name == 'migrate_version':
            continue
        engine.execute(table.delete())


def dummy_context(user=None, project=None, password=None, roles=None,
                  user_id=None, trust_id=None, region_name=None, domain=None,
                  is_admin=False, api_version=None):

    roles = roles or []
    return context.RequestContext.from_dict({
        'project_id': project or 'test_project_id',
        'user_id': user_id or 'test_user_id',
        'user_name': user or 'test_username',
        'password': password or 'password',
        'roles': roles or [],
        'is_admin': is_admin,
        'auth_url': 'http://server.test:5000/v2.0',
        'auth_token': 'abcd1234',
        'trust_id': trust_id or 'trust_id',
        'region_name': region_name or 'region_one',
        'domain_id': domain or '',
        'api_version': api_version or '1.2',
    })


def create_profile(context, profile_id):
    values = {
        'id': profile_id,
        'context': context.to_dict(),
        'type': 'os.nova.server-1.0',
        'name': 'test-profile',
        'spec': {
            'type': 'os.nova.server',
            'version': '1.0',
        },
        'created_at': timeutils.utcnow(True),
        'user': context.user_id,
        'project': context.project_id,
    }
    return objects.Profile.create(context, values)


def create_cluster(context, cluster_id, profile_id, **kwargs):
    values = {
        'id': cluster_id,
        'profile_id': profile_id,
        'name': 'test-cluster',
        'next_index': 1,
        'min_size': 1,
        'max_size': 5,
        'desired_capacity': 3,
        'status': 'ACTIVE',
        'init_at': timeutils.utcnow(True),
        'user': context.user_id,
        'project': context.project_id,
    }
    values.update(kwargs)
    return objects.Cluster.create(context, values)


def create_node(context, node_id, profile_id, cluster_id, physical_id=None):
    values = {
        'id': node_id,
        'name': 'node1',
        'profile_id': profile_id,
        'cluster_id': cluster_id or '',
        'physical_id': physical_id,
        'index': 2,
        'init_at': timeutils.utcnow(True),
        'created_at': timeutils.utcnow(True),
        'role': 'test_node',
        'status': 'ACTIVE',
        'user': context.user_id,
        'project': context.project_id,
    }
    return objects.Node.create(context, values)


def create_policy(context, policy_id, name=None):
    values = {
        'id': policy_id,
        'name': name or 'test_policy',
        'type': 'senlin.policy.dummy-1.0',
        'spec': {
            'type': 'senlin.policy.dummy',
            'version': '1.0',
            'properties': {
                'key1': 'value1',
                'key2': 2
            }
        },
        'created_at': timeutils.utcnow(True),
        'user': context.user_id,
        'project': context.project_id,
    }
    return objects.Policy.create(context, values)
